#region File Header
// <copyright company="Bosch Security Systems GmbH">
//
// Authors:
//    Hubert Streidl<Hubert.Streidl@de.bosch.com>
//    Andreas Wetterich
//
// $Revision: 1.7 $
//
// </copyright>
#endregion

using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Diagnostics;

using FSM5000FSI;
using FSM5000FSIAPI.Version3;

namespace FSISample2
{
   /// <summary>
   /// The given sample application shall demonstrate the handling of command
   /// in the FSI SDK.
   /// 
   /// extensions to sample 1: 
   /// - collection of items and items types
   /// - give the ability to send a command to POINTs (the restrictions to points is only for demonstration)
   /// 
   /// restrictions:
   ///  command parameters are not shown in that sample
   /// 
   /// Configuration:
   /// Adapt the default config values in Main() according to your own  
   /// network configuration.
   /// (The default values in this sample application correspond to the 
   /// example values in the FPA-5000 Networking Guide document.)
   /// </summary>
   /// <param name="args"></param>
   class FSISample2
   {

      /// <summary>
      /// Receive the traces from the FSI and output the traces to console
      /// In real application it is very useful to write the traces to files
      /// for suppport issues
      /// </summary>
      class TraceConsoleWriter : TextWriter
      {
         public override Encoding Encoding
         {
            get { return Encoding.Default; }
         }

         public override void Write(string message)
         {
            Console.Out.Write(message);
         }

         public override void WriteLine(string message)
         {
            Console.Out.WriteLine(message);
         }
      }

      /// <summary>
      /// The Config Listener receives all information about the configuration 
      /// received from the panel network. The information must be stored in real application
      /// to know the devices from the panel network and the possible commands.
      /// 
      /// In contrast to sample 
      /// </summary>
      class ConfigListener : IPanelConfigListener
      {
         /// <summary>
         /// The local store for all items published by the panel
         /// </summary>
         Dictionary<SIID, Item> items = new Dictionary<SIID, Item>();

         public Dictionary<SIID, Item> Items
         {
            get { return items; }            
         }

         /// <summary>
         /// The local store for all item types published by the panel
         /// </summary>
         Dictionary<FunctionType, ItemType> types = new Dictionary<FunctionType, ItemType>();

         public Dictionary<FunctionType, ItemType> Types
         {
            get { return types; }          
         }         

         public void SetItemType(ItemType devItemType)
         {
            Console.Out.WriteLine("Item type " + devItemType.FunctionalType + " (" + devItemType.Description + ")");

            //add item type to local store
            //TODO: use update
            if (!types.ContainsKey(devItemType.FunctionalType))
            {
               types.Add(devItemType.FunctionalType, devItemType);
            }
         }

         public void SetItem(Item devItem)
         {
            Console.Out.WriteLine("Item  (" + devItem.SIID + ", " + devItem.FunctionalType + " (" + devItem.Label + ")");
            //add item to local store 
            //TODO: use UPDATE
            if (!items.ContainsKey(devItem.SIID))
            {
               items.Add(devItem.SIID, devItem);
            }
         }

         public void RemoveItem(SIID address)
         {
            Console.Out.WriteLine("SI " + address + " removed.");
         }
      }

      /// <summary>
      /// The State Listener receive all states for devices send by the panel network.
      /// 
      /// </summary>
      class StateListener : IStateListener
      {
         public void SetItemState(ItemState devItemState)
         {
            Console.Out.WriteLine("State " + devItemState.SIID + " = " + devItemState.CompoundState );
         }

         public void SetMPNetTime(DateTime mpNetTime)
         {
            Console.Out.WriteLine("Time is " + mpNetTime + ".");
         }
      }

      /// <summary>
      /// The main function to start the program
      /// </summary>
      /// <param name="args"></param>
      static void Main(string[] args)
      {
         //Create configuration for FSI 
         FSIConfig cfgFSI = new FSIConfig();

         // ! --- start here to replace values with the values from your own configuration ! ---
         cfgFSI.MPNetGroup = 2;
         cfgFSI.MPNetNode = 1; 
         cfgFSI.PNA = 33;
         cfgFSI.LocalIPAddress = new byte[] { 192, 168, 1, 33 };
         cfgFSI.RepositoryLocation = ".\\repository";

         //  AdvancedFSIConfig is necessary for IP-Multicast settings
         AdvancedFSIConfig advancedFSIConfig = new AdvancedFSIConfig();
         advancedFSIConfig.IsIPMulticast = true;
         advancedFSIConfig.MulticastAddress = new byte[] { 239, 192, 0, 1 };
         advancedFSIConfig.PortNumber = 25001;
         advancedFSIConfig.NetstackTraceLevel = SourceLevels.Error;
         cfgFSI.AdvancedFSIConfig = advancedFSIConfig;
         // ! --- finish here to replace values with the values from your own configuration ! ---

         //Create the instance of the FSI
         FSIController fsi = FSIController.GetInstance();

         //Register the listeners 
         ConfigListener config = new ConfigListener();
         fsi.AddListener(config);
         fsi.AddListener(new StateListener());

         //Activate tracing
         TraceConsoleWriter outputter = new TraceConsoleWriter();
         fsi.SetTraceLevel(SourceLevels.Warning);  //for support issues please set the level to SourceLevels.Verbose
         fsi.AddTraceTextWriter(outputter);
        
         //Startup with configuration
         Result result = fsi.Startup(cfgFSI);

         Console.Out.WriteLine("->>>>>>>> Startup=" + result);

         ShowUserInputText();

         ConsoleKeyInfo input;
         bool run = true;
         do
         {
            input = Console.ReadKey();
            switch (input.Key)
            {
               case ConsoleKey.E:
                  run = false; // finish the application
                  break;
               case ConsoleKey.C:
                  SendCommandToPanel(fsi, config); //demonstrate the sending of commands
                  ShowUserInputText();
                  break;
            }
         }
         while (run);


         //shutdown the sample application
         fsi.Shutdown();
      

      }
    
      /// <summary>
      /// The methode demostrate the sending of commands to the panel network. The following steps are required.
      /// 1. Choose an item for the command
      /// 2. Choose a command supported by the function type of the command
      /// 3. Create and send the command to the panel  
      /// </summary>
      /// <param name="fsi"></param>
      /// <param name="config"></param>
      private static void SendCommandToPanel(FSIController fsi, ConfigListener config)
      {
         ///show items to send commands
         Console.Out.WriteLine();
         short itemIndex = 0;
         List<Item> items = new List<Item>();
         Console.Out.WriteLine("Index | Item SIID");
         foreach (Item item in config.Items.Values)
         {
            if (item.SIID.SIType == SIType.SI_POINT)
            {               
               Console.Out.WriteLine("{0}     | Point: {1}", itemIndex, item.SIID);
               items.Add(item);
               itemIndex++;
            }
         }

         ///Select an item
         Console.Out.WriteLine("Input the item index and press ENTER to show available commands");
         String inputSelectedItem = Console.In.ReadLine();
         short selectedItemIndex;
         if (Int16.TryParse(inputSelectedItem, out selectedItemIndex) && selectedItemIndex < items.Count)
         {              
            ///Now we have the item for that the command shall be send
            Item theItem = items[selectedItemIndex];
            
            ItemType type;            
            if (config.Types.TryGetValue(theItem.FunctionalType, out type))
            {               
               Console.Out.WriteLine("Is type: " + type.Description + " + support the commands");
               ///Show the command list for the functiontype of the item
               short commandIndex = 0;
               Console.Out.WriteLine("Index | Command");
               foreach (ItemTypeCommand cmd in type.ItemTypeCommands)
               {
                  Console.Out.WriteLine("{0}     | {1}",commandIndex++, cmd.DisplayName);
               }

               ///Select a command for the item
               Console.Out.WriteLine("Press the number of command and ENTER to execute command for item");
               String inputSelectedCommand = Console.In.ReadLine();
               short selectedCommandIndex;
               if (Int16.TryParse(inputSelectedCommand, out selectedCommandIndex) && selectedCommandIndex < type.ItemTypeCommands.Length)
               {
                  ItemTypeCommand theCommand = type.ItemTypeCommands[selectedCommandIndex];
                  Console.Out.WriteLine("Execute command: '" + theCommand.Description + "' for '" + theItem.SIID + "'");

                  //Create the command
                  //for the command string only the values from ItemTypeCommand.Name must be used !!!
                  PanelCommand command = new PanelCommand(theItem.SIID, theCommand.Name);

                  //Send the command to the panel
                  fsi.ReceiveCommand(command);
               }
               else
               {
                  Console.Out.WriteLine("Invalid Input");
               }
            }

         }
         else
         {
            Console.Out.WriteLine("Invalid Input");
         }
      }

      /// <summary>
      /// helper methode to show the user input text
      /// </summary>
      private static void ShowUserInputText()
      {
         //user interaction to finish the application
         Console.Out.WriteLine("======================================================================");
         Console.Out.WriteLine("Press e to end the sample application");
         Console.Out.WriteLine("Press c to send an command");
         Console.Out.WriteLine("======================================================================");
      }

   }
}
