#region File Header
// <copyright company="Bosch Security Systems GmbH">
//
// Authors:
//    Hubert Streidl<Hubert.Streidl@de.bosch.com>
//    Andreas Wetterich
//
// $Revision: 1.5 $
//
// </copyright>
#endregion

using System;
using System.Runtime.Remoting.Lifetime;

//the api version used in this sample
using FSM5000FSIAPI.Version3;

namespace FSISample5
{

   /// <summary>
   /// Sample FSISample5 demonstrates how to run more than one FSIInstance at the same time: 
   ///
   /// The program allows the user to enter startup settings from a console window and 
   /// starts up to 4 instances of FSI lib. Each instance writes traces into different
   /// trace files. 3 output files are created for each instance:
   /// - library traces
   /// - traces in the PanelConfigListener
   /// - traces in the StateListener
   /// 
   /// The sample may also be started on different PCs simultaneously in order to create 
   /// dual homing and redundancy scenarios for test purpose.
   /// 
   /// Pre-conditions to build and run the sample:
   /// - Copy the content of the SDKs '\bin' directory into the '.\sdk' directory to run the 
   ///   sample.
   /// - If necessary remove all versions of FSM5000FSI from the directory except the one that
   ///   matches the version of your test panel.
   /// - Adopt the references to FSM5000FSILoader and FSM5000FSIAPI in the project file 
   ///   (both files are located in \sdk after copying)
   /// - Adapt the default config values in the Configurator() according to your own  
   ///   network configuration.
   ///   (The default values in this sample application correspond to the 
   ///   example values in the FPA-5000 Networking Guide document.)
   /// </summary>

   class FSISample5
   {
      const byte NumberOfConnections = 4;

      static NetworkConnection [] Connections;


      /// <summary>
      /// The Config Listener receives all information about the configuration 
      /// received from the panel network. The information must be stored in real application
      /// to know the devices from the panel network and the possible commands.
      /// </summary>
      class ConfigListener : MarshalByRefObject, IPanelConfigListener
      {
         public override object InitializeLifetimeService()
         {
            ILease lease = (ILease)base.InitializeLifetimeService();
            if (lease.CurrentState == LeaseState.Initial)
            {
               lease.InitialLeaseTime = TimeSpan.Zero;
            }
            return lease;
         }

         public void SetItemType(ItemType devItemType)
         {
            Console.Out.WriteLine("Item type " + devItemType.FunctionalType + " (" + devItemType.Description + ")");
         }

         public void SetItem(Item devItem)
         {
            Console.Out.WriteLine("Item  (" + devItem.SIID + ", " + devItem.FunctionalType + " (" + devItem.Label + ")");
         }

         public void RemoveItem(SIID address)
         {
            Console.Out.WriteLine("SI " + address + " removed.");
         }
      }

      /// <summary>
      /// The State Listener receives all states for devices sent by the panel network.
      /// 
      /// </summary>
      class StateListener : MarshalByRefObject, IStateListener
      {
         public override object InitializeLifetimeService()
         {
            ILease lease = (ILease)base.InitializeLifetimeService();
            if (lease.CurrentState == LeaseState.Initial)
            {
               lease.InitialLeaseTime = TimeSpan.Zero;
            }
            return lease;
         }

         public void SetItemState(ItemState devItemState)
         {
            Console.Out.WriteLine("State " + devItemState.SIID + " = " + devItemState.LogicalState);
         }

         public void SetMPNetTime(DateTime mpNetTime)
         {
            Console.Out.WriteLine("Time is " + mpNetTime + ".");
         }
      }


      static void Main(string[] args)
      {
         Connections = new NetworkConnection[NumberOfConnections];
         for (int i = 0; i < NumberOfConnections; i++)
         {
            Connections[i] = new NetworkConnection();
         }

         try
         {
            CommandShell();
         }
         catch (SystemException ex)
         {
            Console.Out.WriteLine("Exception: " + ex.Message);
         }
      }


      static private void CommandShell()
      {
         //user interaction to finish the application
         Prompt();

         ConsoleKeyInfo input;
         do
         {
            input = Console.ReadKey();
            Console.Out.WriteLine();

            switch (input.Key)
            {
               //print configurations
               case ConsoleKey.A:
               {
                  PrintConfigurations();
               }
               break;

               //print status of connections
               case ConsoleKey.B:
               { 
                  PrintStatusOfConnections();
               }
               break;

               //change configuration of connection
               case ConsoleKey.C:
               { 
                  ChangeConfiguration();
               }
               break;

               //exit program
               case ConsoleKey.E:
               { 
                  return;
               }

               //help output
               case ConsoleKey.H:
               { 
                  Help();
               }
               break;

               //startup connection
               case ConsoleKey.S:
               { 
                  StartupConnection();
               }
               break;

               //shutdown connection
               case ConsoleKey.X:
               { 
                  ShutdownConnection();
               }
               break;

               default:
               {
                  Console.Out.WriteLine("Invalid command");
               }
               break;
            }

            Prompt();
         }
         while (input.Key != ConsoleKey.E);
      }

      static private void Prompt()
      {
         Console.Out.WriteLine();
         Console.Out.WriteLine("=========== FSI Sample (Press h to print help) ==================");
         Console.Out.Write("> ");
      }

      static private void Help()
      {
         Console.Out.WriteLine("a - print configurations");
         Console.Out.WriteLine("b - print status of connections");
         Console.Out.WriteLine("c - change configuration of connection");
         Console.Out.WriteLine("e - exit program");
         Console.Out.WriteLine("h - print this help");
         Console.Out.WriteLine("s - Startup connection");
         Console.Out.WriteLine("x - Shutdown connection");
      }

      static private void PrintConfigurations()
      {
         Console.Out.WriteLine("--- Configuration of connections ----");
         for (int i = 0; i < NumberOfConnections; i++)
         {
            string output = string.Format("---- Connection #{0} ----", i);
            Console.Out.WriteLine(output);
            Connections[i].PrintCurrentConfig();
            Console.Out.WriteLine();
         }
      }

      static private void PrintStatusOfConnections()
      {
         Console.Out.WriteLine("--- Status of connections ----");
         for (int i = 0; i < NumberOfConnections; i++)
         {
            string output = string.Format("Connection #{0}: {1}", i, Connections[i].StatusDescription);
            Console.Out.WriteLine(output);
         }
      }

      static private void ChangeConfiguration()
      {
         ushort connection = SelectConnection();
         if (connection < NumberOfConnections)
         {
            Connections[connection].ReadConfig();
         }
      }

      static private void StartupConnection()
      {
         ushort connection = SelectConnection();
         if (connection < NumberOfConnections)
         {
            Connections[connection].Startup();
         }
      }

      static private void ShutdownConnection()
      {
         ushort connection = SelectConnection();
         if (connection < NumberOfConnections)
         {
            Connections[connection].Shutdown();
         }
      }

      static ushort SelectConnection()
      {
         ushort num = NumberOfConnections;
         string output = string.Format("   Enter the connection number (0 ... {0})", NumberOfConnections-1);
         Console.Out.WriteLine(output);

         string input = Console.ReadLine();
         if (ushort.TryParse(input, out num))
         {
            if (false == (num < NumberOfConnections))
            {
               Console.Out.WriteLine("Required number must be in range 0 ... {0}", NumberOfConnections - 1);
            }
         }
         else
         {
            Console.Out.WriteLine("Invalid input");
         }

         return num;
      }

   }
}
