#region File Header
// <copyright company="Bosch Security Systems GmbH">
//
// Authors:
//    Hubert Streidl<Hubert.Streidl@de.bosch.com>
//    Andreas Wetterich
//
// $Revision: 1.5 $
//
// </copyright>
#endregion

using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Diagnostics;

using FSM5000FSI;
using FSM5000FSIAPI.Version3;

namespace FSISample6
{
   /// <summary>
   /// The given sample application shall demonstrate the handling of command
   /// in the FSI SDK.
   /// 
   /// extensions to sample 1: 
   /// - collection of items and items types
   /// - give the ability to send a command to POINTs (the restrictions to points is only for demonstration)
   /// 
   /// restrictions:
   ///  command parameters are not shown in that sample
   /// 
   /// Configuration:
   /// Adapt the default config values in Main() according to your own  
   /// network configuration.
   /// (The default values in this sample application correspond to the 
   /// example values in the FPA-5000 Networking Guide document.)
   /// </summary>
   /// <param name="args"></param>
   class FSISample6
   {

      /// <summary>
      /// Receive the traces from the FSI and output the traces to console
      /// In real application it is very useful to write the traces to files
      /// for suppport issues
      /// </summary>
      class TraceConsoleWriter : TextWriter
      {
         public override Encoding Encoding
         {
            get { return Encoding.Default; }
         }

         public override void Write(string message)
         {
            Console.Out.Write(message);
         }

         public override void WriteLine(string message)
         {
            Console.Out.WriteLine(message);
         }

      }

      /// <summary>
      /// The Config Listener receives all information about the configuration 
      /// received from the panel network. The information must be stored in real application
      /// to know the devices from the panel network and the possible commands.
      /// 
      /// In contrast to sample 
      /// </summary>
      class ConfigListener : IPanelConfigListener
      {

         public void SetItemType(ItemType devItemType)
         {
            Console.Out.WriteLine("Item type " + devItemType.FunctionalType + " (" + devItemType.Description + ")");
         }

         public void SetItem(Item devItem)
         {
            Console.Out.WriteLine("Item  (" + devItem.SIID + ", " + devItem.FunctionalType + " (" + devItem.Label + ")");
         }

         public void RemoveItem(SIID address)
         {
            Console.Out.WriteLine("SI " + address + " removed.");
         }
      }

      /// <summary>
      /// The State Listener receive all states for devices send by the panel network.
      /// 
      /// </summary>
      class StateListener : IStateListener
      {
         /// <summary>
         /// stores item states for events which reqires request
         /// </summary>
         Dictionary<SIID, ItemState> items = new Dictionary<SIID, ItemState>();

         public Dictionary<SIID, ItemState> Items
         {
            get { return items; }
         }

         public void SetItemState(ItemState devItemState)
         {
            //sequential search to find out whether the value is already in the store
            foreach (SIID siid in items.Keys)
            {
               if (siid.Equals(devItemState.SIID))
               {
                  Items.Remove(siid);
                  break;
               }
            }

            if (devItemState.Classification == Classification.OPERATOR_REQUEST)
            {
               Items.Add(devItemState.SIID, devItemState);
            }

            Console.Out.WriteLine("State " + devItemState.SIID + " = " + devItemState.CompoundState);
         }

         public void SetMPNetTime(DateTime mpNetTime)
         {
            Console.Out.WriteLine("Time is " + mpNetTime + ".");
         }
      }

      /// <summary>
      /// The main function to start the program
      /// </summary>
      /// <param name="args"></param>
      static void Main(string[] args)
      {
         //Create configuration for FSI 
         FSIConfig cfgFSI = new FSIConfig();

         // ! --- start here to replace values with the values from your own configuration ! ---
         cfgFSI.MPNetGroup = 2; // ! replace with the values in you configuration !
         cfgFSI.MPNetNode = 1; // ! replace with the values in you configuration !
         cfgFSI.PNA = 33; // ! replace with the values in you configuration !         
         cfgFSI.LocalIPAddress = new byte[] { 192, 168, 1, 33 }; // ! replace with the values in you configuration !
         cfgFSI.RepositoryLocation = ".\\repository";

         //  AdvancedFSIConfig is necessary for IP-Multicast settings
         AdvancedFSIConfig advancedFSIConfig = new AdvancedFSIConfig();
         advancedFSIConfig.IsIPMulticast = true;
         advancedFSIConfig.MulticastAddress = new byte[] { 239, 192, 0, 1 };
         advancedFSIConfig.PortNumber = 25001;
         advancedFSIConfig.NetstackTraceLevel = SourceLevels.Error;
         cfgFSI.AdvancedFSIConfig = advancedFSIConfig;
         // ! --- finish here to replace values with the values from your own configuration ! ---

         //Create the instance of the FSI
         FSIController fsi = FSIController.GetInstance();

         //Register the listeners 
         ConfigListener config = new ConfigListener();
         fsi.AddListener(config);
         StateListener states = new StateListener();
         fsi.AddListener(states);

         //Activate tracing
         TraceConsoleWriter outputter = new TraceConsoleWriter();
         fsi.SetTraceLevel(SourceLevels.Warning);  //for support issues please set the level to SourceLevels.Verbose
         fsi.AddTraceTextWriter(outputter);
        
         //Startup with configuration
         Result result = fsi.Startup(cfgFSI);

         Console.Out.WriteLine("->>>>>>>> Startup=" + result);

         ShowUserInputText();

         ConsoleKeyInfo input;
         bool run = true;
         do
         {
            input = Console.ReadKey();
            switch (input.Key)
            {
               case ConsoleKey.E:
                  run = false; // finish the application
                  break;
               case ConsoleKey.C:
                  SendCommandToPanel(fsi, states); //demonstrate the sending of commands
                  ShowUserInputText();
                  break;
            }
         }
         while (run);


         //shutdown the sample application
         fsi.Shutdown();
      

      }
    
      /// <summary>
      /// The methode demostrate the sending of commands to the panel network. The following steps are required.
      /// 1. Choose an item for the command
      /// 2. Choose a command supported by the function type of the command
      /// 3. Create and send the command to the panel  
      /// </summary>
      /// <param name="fsi"></param>
      /// <param name="config"></param>
      private static void SendCommandToPanel(FSIController fsi, StateListener states)
      {
         ///show items to send commands
         Console.Out.WriteLine();
         short itemIndex = 0;
         List<ItemState> items = new List<ItemState>();
         Console.Out.WriteLine("Index | Item SIID");
         foreach (ItemState itemState in states.Items.Values)
         {
            Console.Out.WriteLine("{0}     | ItemState: {1} {2} {3}", itemIndex, itemState.SIID, itemState.CompoundState, itemState.StateOfHandling);
            items.Add(itemState);
            itemIndex++;
         }

         ///Select an item
         Console.Out.WriteLine("Input the item index:");
         Console.Out.WriteLine("    > ");
         String inputSelectedItem = Console.In.ReadLine();
         short selectedItemIndex;
         if (Int16.TryParse(inputSelectedItem, out selectedItemIndex) && selectedItemIndex < items.Count)
         {
            ItemState selectedItem = items[selectedItemIndex];
            Console.Out.WriteLine("Press either 'a' (Acknowledge) or 'r' (Reset). ");
            Console.Out.WriteLine("    > ");
            ConsoleKeyInfo key = Console.ReadKey();

            if( (key.Key != ConsoleKey.A) || (key.Key != ConsoleKey.R) )
            {
               //Note: this is an example for operator handling commands, so we are using a short cut to get the command name. This approach 
               //is for the sake of simplicity in this sample (we know the name and hard-code it.) but it is not the recommended way.
               //The correct approach would be to extract the commands from the ItemTypeCommand list in the function type definition, by
               //showing the description texts for each command to the user and take the command name from the command definition
               //which is selected by the user (as demonstrated in FSISample2)
               if(key.Key == ConsoleKey.A)
               {
                  //send acknowledge
                  PanelCommand command = new PanelCommand(selectedItem.SIID, "ACK", selectedItem.SequenceNumber);

                  //Send the command to the panel
                  fsi.ReceiveCommand(command);
               }
               else
               {
                  //create reset command
                  PanelCommand command = new PanelCommand(selectedItem.SIID, "RESET", selectedItem.SequenceNumber);
                  
                  //Send the command to the panel
                  fsi.ReceiveCommand(command);
               }
            }
            else
            {
               Console.Out.WriteLine("Invalid Input");
            }
         }
         else
         {
            Console.Out.WriteLine("Invalid Input");
         }
      }

      /// <summary>
      /// helper methode to show the user input text
      /// </summary>
      private static void ShowUserInputText()
      {
         //user interaction to finish the application
         Console.Out.WriteLine("======================================================================");
         Console.Out.WriteLine("Press e to end the sample application");
         Console.Out.WriteLine("Press c to send an command");
         Console.Out.WriteLine("======================================================================");
      }

   }
}
